# Introduction
Below is a list of project ideas for [GSoC](https://www.google-melange.com/gsoc/homepage/google/gsoc2014) 2014. All these projects require familiarity with the Nimrod programming, or at minimum, experience with similar programming languages such as C, C#, Java, Python, etc.

**Note** - Any ideas to be added to this page must be placed in a relevant category, have a title, a line stating needed/preferred skills for the idea, and a summary of what the idea is.

# Projects

## Nimrod Compiler
#### Add support for full coroutines
  * Full coroutines in contrast to Nimrod's closure iterators can capture the full call stack.
  * Needs inline assembler to implement the stack switching.
  * The GC needs to support conservative marking of multiple stacks
  * In particular the write barrier in the GC which does the 'isOnStack' check needs to be changed.
  * Using a bloom filter for quick testing whether an address belongs to some stack is likely to pay off.
  * Creating a coroutine needs to register a new stack to the GC.
  * Deleting a coroutine needs to unregister the stack to the GC.
  * Builtin 'yld' must save the current stack pointer so that the GC knows which part of the stack is really in use.

#### Add a code generator for OpenCL

#### Fix bugs with iterators/generics
...

#### Fix/Expand Nimrod's "Compiler as a Service" features
**Skill Requirements:** Command line argument parsing, minimal knowledge of compiler internals and theory.
**Description:**
  * Fix https://github.com/Araq/Nimrod/issues/804, which makes `idetools` unusable.
  * Provide commands to execute macros. Allow IDEs to quickly get the code macros generate with different inputs.
  * Allow the compiler to either track more than one project at a time, or not track any project at all.

**Bonus points:** Implement "Compiler as a Service" support to major IDEs/text editors such as Light Table, Sublime Text, or Visual Studio.

#### Allow bootstrap-time integration of the Nimrod executable with the standard library
**Skill Requirements:** Knowledge of file system api's, knowledge of compiler internals and theory.
**Description**:
  * Allow the nimrod bootstrap process to integrate the nimrod standard library source code and other associated resources into the nimrod binary, to be used by the nimrod binary when compiling source code. The included library modules should be overridable, either by a switch passed to the nimrod executable, or by placing an actually library file in a pre-determined path.


## Standard Library
####Integrate and expand new async io module

#### Enhance the filesystem monitoring module "fsmonitor.nim"
**Skill Requirements:** Knowledge of the Microsoft Windows api.
**Description**:
  * Allow the fsmonitor module to work on Microsoft Windows by using native api's to gather information about changes in monitored files and directories.
  * Revise the fsmonitor module api to decouple Unix/Linux file handle paradigms (such as using the poll method in in the sockets module) from the api, allowing easier implementations of multiple native backends.
  * Integrate the fsmonitor module's polling mechanism into the new asynchronous io modules.

#### Add a cross-platform stat()-like procedure to the operating system module "os.nim"
**Skill Requirements:** Knowledge of file system api's for Linux, MacOSX, or Microsoft Windows.
**Description:**
  * Implement a procedure which uses native stat-like calls on Linux, Mac, Windows, and other operating systems to gather detailed information about specific file system objects. Allow the bypassing of symlinks and hardlinks, where possible.

#### Enhance and expand standard library documentation
**Skill Requirements:** Basic writing and documentation skills, web design and infrastructure setup.
**Description**:
  * Ensure that documentation exists for all public methods and modules.
  * Create and design new CSS and HTML layouts for the documentation, to better fit with the main website.
  * Add search capabilities to the online documentation.

#### Add documentation to the nimrod compiler internals
**Skill Requirements:** Basic writing and documentation skills.
**Description:**
  * Add comments to the compiler internals, documenting the various mechansisms and mechanics the compiler uses to analyze and transform nimrod code to the code of the specified backend.

#### Improve times module
**Skill Requirements** Knowledge of date time representations, native time api's.
**Description**:
  * Fix limitations to do with time intervals, specifically subtracting a TTimeInterval from a TTimeInfo.
  * Provide a ``$`` for TTimeInterval. Goal is to be able to get timing info like "5 minutes ago".

#### Add an implementation of the ISAAC psuedorandom number generator
**Skill Requirements** Pseudo-Random number generation theory, C programming.
**Description:**
  * Create a pure-nimrod implementation of the [[ISAAC Random Number Generator|http://burtleburtle.net/bob/rand/isaacafa.html]] .

#### Wrap and test the Qt framework


## Tools & Infrastructure
#### Update and refactor nimrod builder
**Skill Requirements**: JSON parsing, modular program construction, inter-process communication.
**Description**:
  * Reduce the number of assumptions the nimrod builder makes about its host system, in order to reduce configuration restrictions. Assumptions include location and usage of external tools, such as git.

#### Implement re2nim, a lexer generator for nimrod
  * Model it after re2c or the Ragel state machine generator

#### Implement a Nimrod backend for the Ragel state machine generator
  * http://www.complang.org/ragel/
